searchData={"items":[{"type":"behaviour","title":"Dotenvy","doc":"`Dotenvy` is an Elixir port of the original [dotenv](https://github.com/bkeepers/dotenv) Ruby gem.\n\nIt is designed to help applications follow the principles of\nthe [12-factor app](https://12factor.net/) and its recommendation to store\nconfiguration in the environment.\n\nUnlike other configuration helpers, `Dotenvy` enforces no convention for the naming\nof your files: `.env` is a common choice, you may name your configuration files whatever\nyou wish.\n\nSee the [strategies](docs/strategies.md) for examples of various use cases.","ref":"Dotenvy.html"},{"type":"function","title":"Dotenvy.env/3","doc":"","ref":"Dotenvy.html#env/3"},{"type":"function","title":"Dotenvy.env!/2","doc":"Reads the given env `variable` and converts its value to the given `type`.\n\nThis function attempts to read a value from a local data store of sourced values.\n\nThis function may raise an error because type conversion is delegated to\n`Dotenvy.Transformer.to!/2` -- see its documentation for a list of supported types.","ref":"Dotenvy.html#env!/2"},{"type":"function","title":"Examples - Dotenvy.env!/2","doc":"iex> env!(\"PORT\", :integer)\n    5432\n    iex> env!(\"ENABLED\", :boolean)\n    true","ref":"Dotenvy.html#env!/2-examples"},{"type":"function","title":"Dotenvy.env!/3","doc":"Reads an env variable and converts its output or returns a default value.\n\n> #### Use `env!/2` when possible {: .info}\n>\n> Use of `env!/2` is recommended over `env!/3` because it creates a stronger contract with\n> the environment: your app literally will not start when required env variables are missing.\n\nIf the given `variable` is *set*, its value is converted to\nthe given `type`. The provided `default` value is _only_ used when the\nvariable is _not_ set; **the `default` value is returned as-is, without conversion**.\nThis allows greater control of the output.\n\nConversion is delegated to `Dotenvy.Transformer.to!/2`, which may raise an error.\nSee its documentation for a list of supported types.\n\nThis function attempts to read a value from a local data store of sourced values.","ref":"Dotenvy.html#env!/3"},{"type":"function","title":"Examples - Dotenvy.env!/3","doc":"iex> env!(\"PORT\", :integer, 5432)\n    5433\n\n    iex> env!(\"NOT_SET\", :boolean, %{not: \"converted\"})\n    %{not: \"converted\"}\n\n    iex> System.put_env(\"HOST\", \"\")\n    iex> env!(\"HOST\", :string!, \"localhost\")\n    ** (RuntimeError) Error converting HOST to string!: non-empty value required","ref":"Dotenvy.html#env!/3-examples"},{"type":"callback","title":"Dotenvy.parse/3","doc":"A parser implementation should receive the `contents` read from a file,\na map of `vars` (with string keys, as would come from `System.get_env/0`),\nand a keyword list of `opts`.\n\nThis callback is provided to help facilitate testing. See `Dotenvy.Parser`\nfor the default implementation.","ref":"Dotenvy.html#c:parse/3"},{"type":"function","title":"Dotenvy.source/2","doc":"Like its Bash namesake command, `source/2` accumulates values from the given input(s).\nThe accumulated values are stored via a side effect function to make them available\nto the `env!/2` and `env!/3` functions.\n\nThink of `source/2` as a _merging operation_ which can accept maps (like `Map.merge/2`)\nor paths to env files.\n\nInputs are processed from left to right so that values can be overridden by each\nsubsequent input. As with `Map.merge/2`, the right-most input takes precedence.","ref":"Dotenvy.html#source/2"},{"type":"function","title":"Options - Dotenvy.source/2","doc":"- `:parser` module that implements `c:Dotenvy.parse/3` callback. Default: `Dotenvy.Parser`\n\n- `:require_files` specifies which of the given `files` (if any) *must* be present.\n  When `true`, all the listed files must exist.\n  When `false`, none of the listed files must exist.\n  When some of the files are required and some are optional, provide a list\n  specifying which files are required. If a file listed here is not included\n  in the function's `files` argument, it is ignored. Default: `false`\n\n- `:side_effect` an arity 1 function called after the successful parsing inputs.\n  The default is an internal function that stores the values inside a process dictionary so\n  the values are available to the `env!/2` and `env!/3` functions. This option\n  is overridable to facilitate testing. Changing it is not recommended.","ref":"Dotenvy.html#source/2-options"},{"type":"function","title":"Examples - Dotenvy.source/2","doc":"The simplest implementation is to parse a single file by including its path:\n\n    iex> Dotenvy.source(\".env\")\n    {:ok, %{\n      \"TIMEOUT\" => \"5000\",\n      \"DATABASE_URL\" => \"postgres://postgres:postgres@localhost/myapp\",\n      # ...etc...\n      }\n    }\n\nMore commonly, you will source _multiple_ files (often based on the `config_env()`)\nand you will defer to pre-existing system variables. The most common pattern looks like this:\n\n      iex> Dotenvy.source([\n        \"#{config_env()}.env\",\n        \"#{config_env()}.override.env\",\n        System.get_env()\n      ])\n\nIn the above example, the `prod.env`, `dev.env`, and `test.env` files would be version-controlled,\nbut the `*.override.env` variants would be ignored, giving developers the ability to override\nvalues without needing to modify versioned files.\n\n> #### Give Precedence to System Envs! {: .warning}\n>\n> Don't forget to include `System.get_env()` as the final input to `source/2` so that\n> system environment variables take precedence over values sourced from `.env` files.\n>\n> When you run a shell command like `❯ LOG_LEVEL=debug mix run`, your expectation is probably that\n> the `LOG_LEVEL` variable would be set to `debug`, overriding whatever may have been defined\n> in your sourced `.env` files. Similarly, you may export env vars in your Bash profile.\n> System env vars are not granted precedence automatically: you must explicitly include\n> `System.get_env()` as the final input to `source/2`.\n\n\nIf your env files are making use of variable substitution based on system env vars,\ne.g. `${PWD}` (see the [Dotenv File Format](docs/dotenv-file-format.md)), then you\nwould need to specify `System.get_env()` as the first argument to `source/2`.\n\nFor example, if your `.env` references the system `HOME` variable:\n  ```\n  # .env\n  CACHE_DIR=${HOME}/cache\n  ```\n\nthen your `source/2` command would need to make the system env vars available\nto the parser by including them as one of the inputs, e.g.\n\n      iex> Dotenvy.source([System.get_env(), \".env\"])\n\nIncluding the `System.get_env()` before your files means that your files have final\nsay over the values, potentially overriding any pre-existing system env vars. In\nsome cases, you may wish to reference the system vars both _before and after_ your own\n.env files, e.g.\n\n      iex> Dotenvy.source([System.get_env(), \".env\", System.get_env()])\n\nor you may wish to cherry-pick which variables you need to make available for\nvariable substitution:\n\n      iex> Dotenvy.source([\n        %{\"HOME\" => System.get_env(\"HOME\")},\n        \".env\",\n        System.get_env()\n      ])\n\nThis syntax favors explicitness so there is no confusion over what might have been\n\"automagically\" accumulated.","ref":"Dotenvy.html#source/2-examples"},{"type":"function","title":"Dotenvy.source!/2","doc":"As `source/2`, but returns a map on success or raises on error.","ref":"Dotenvy.html#source!/2"},{"type":"type","title":"Dotenvy.input_source/0","doc":"An input source may be either a path to an env file or a map with string keys,\ne.g. `\"envs/.env\"` or `%{\"FOO\" => \"bar\"}`. This allows users to specify a list\nof env files interspersed with other values from other sources, e.g. `System.get_env()`\nor values fetched from a secure parameter store.","ref":"Dotenvy.html#t:input_source/0"},{"type":"module","title":"Dotenvy.Parser","doc":"This module handles the parsing of the contents of `.env` files into maps with\nstring keys. See [Dotenv File Format](docs/dotenv-file-format.md) for details\non the supported file format.\n\nThis implementation uses parsing over regular expressions for most of its work.","ref":"Dotenvy.Parser.html"},{"type":"function","title":"Dotenvy.Parser.parse/3","doc":"Parse the given `contents`, substituting and merging with the given `vars`.","ref":"Dotenvy.Parser.html#parse/3"},{"type":"module","title":"Dotenvy.Transformer","doc":"This module provides functionality for converting string values to specific Elixir data types.\n\nThese conversions were designed to operate on system environment variables, which\n_always_ store string binaries.","ref":"Dotenvy.Transformer.html"},{"type":"function","title":"Dotenvy.Transformer.to!/2","doc":"Converts strings into Elixir data types with support for nil-able values. Raises on error.\n\nEach type determines how to interpret the incoming string, e.g. when the `type`\nis `:integer`, an empty string is considered a `0`; when `:integer?` is the `type`,\nand empty string is converted to `nil`.\n\nRemember:\n\n- Use a `?` suffix when an empty string should be considered `nil` (a.k.a. a \"nullable\" value).\n- Use a `!` suffix when an empty string is not allowed. Use this when values are required.","ref":"Dotenvy.Transformer.html#to!/2"},{"type":"function","title":"Types - Dotenvy.Transformer.to!/2","doc":"See the `t:Dotenvy.Transformer.conversion_type/0` for a description of valid\nconversion types.","ref":"Dotenvy.Transformer.html#to!/2-types"},{"type":"function","title":"Examples - Dotenvy.Transformer.to!/2","doc":"iex> to!(\"debug\", :atom)\n    :debug\n    iex> to!(\"\", :boolean)\n    false\n    iex> to!(\"\", :boolean?)\n    nil\n    iex> to!(\"5432\", :integer)\n    5432\n    iex> to!(\"foo\", fn val -> val <> \"bar\" end)\n    \"foobar\"","ref":"Dotenvy.Transformer.html#to!/2-examples"},{"type":"type","title":"Dotenvy.Transformer.conversion_type/0","doc":"The conversion type specifies the target data type to which a string will be converted.\nFor example, `:integer` would indicate a transformation of `\"12\"` to `12`.\n\nThe following types are supported:\n\n- `:atom` - converts to an atom. An empty string will be the atom `:\"\"` (!).\n- `:atom?` - converts to an atom. An empty string will be considered `nil`\n- `:atom!` - converts to an atom. An empty string will raise.\n\n- `:boolean` - \"false\", \"0\", or an empty string \"\" will be considered boolean `false`. Any other non-empty value is considered `true`.\n- `:boolean?` - as above, except an empty string will be considered `nil`\n- `:boolean!` - as above, except an empty string will raise.\n\n- `:charlist` - converts string to charlist.\n- `:charlist?` - converts string to charlist. Empty string will be considered `nil`.\n- `:charlist!` - as above, but an empty string will raise.\n\n- `:integer` - converts a string to an integer. An empty string will be considered `0`.\n- `:integer?` - as above, but an empty string will be considered `nil`.\n- `:integer!` - as above, but an empty string will raise.\n\n- `:float` - converts a string to an float. An empty string will be considered `0`.\n- `:float?` - as above, but an empty string will be considered `nil`.\n- `:float!` - as above, but an empty string will raise.\n\n- `:existing_atom` - converts into an existing atom. Raises error if the atom does not exist.\n- `:existing_atom?` - as above, but an empty string will be considered `nil`.\n- `:existing_atom!` - as above, but an empty string will raise.\n\n- `:module` - converts a string into an Elixir module name. Raises on error.\n- `:module?` - as above, but an empty string will be considered `nil`.\n- `:module!` - as above, but an empty string will raise.\n\n- `:string` - no conversion (default)\n- `:string?` - empty strings will be considered `nil`.\n- `:string!` - as above, but an empty string will raise.\n- custom function - see below.","ref":"Dotenvy.Transformer.html#t:conversion_type/0"},{"type":"type","title":"Custom Callback function - Dotenvy.Transformer.conversion_type/0","doc":"When you require more control over the transformation of your value than is possible\nwith the types provided, you can provide an arity 1 function in place of the type.","ref":"Dotenvy.Transformer.html#t:conversion_type/0-custom-callback-function"},{"type":"exception","title":"Dotenvy.Error","doc":"This error module can be useful when writing your own custom conversion\nfunctions because special contextual information will be included with any\nerrors.","ref":"Dotenvy.Error.html"},{"type":"exception","title":"Examples - Dotenvy.Error","doc":"Let's say your configuration needs to supply one of a set of possible values\n(i.e. an enum). We can define a custom function to support this and pass it\nas the second argument to `Dotenvy.env!/2`\n\n    # runtime.exs\n    import Config\n    import Dotenvy\n\n    size_enum = fn\n      \"large\" -> :large\n      \"small\" -> :small\n      _ ->\n        raise Dotenvy.Error, message: \"allowed size_enum values are large or small\"\n    end\n\n    config :myapp, :some_bool, env!(\"SIZE\", size_enum)","ref":"Dotenvy.Error.html#module-examples"},{"type":"extras","title":"Dotenvy","doc":"# Dotenvy\n\n[![Module Version](https://img.shields.io/hexpm/v/dotenvy.svg)](https://hex.pm/packages/dotenvy)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/dotenvy/)\n[![Total Download](https://img.shields.io/hexpm/dt/dotenvy.svg)](https://hex.pm/packages/dotenvy)\n[![License](https://img.shields.io/hexpm/l/dotenvy.svg)](https://hex.pm/packages/dotenvy)\n[![Last Updated](https://img.shields.io/github/last-commit/fireproofsocks/dotenvy.svg)](https://github.com/fireproofsocks/dotenvy/commits/master)\n\n`Dotenvy` is an Elixir port of the original [dotenv](https://github.com/bkeepers/dotenv) Ruby gem, **compatible with mix and releases**. It is designed to help the development of applications following the principles of the [12-factor app](https://12factor.net/) and its recommendation to store configuration in the environment.","ref":"readme.html"},{"type":"extras","title":"Installation - Dotenvy","doc":"Add `dotenvy` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:dotenvy, \"~> 0.9.0\"}\n  ]\nend\n```\n\nIt has no dependencies.","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Dotenvy","doc":"`Dotenvy` is designed to help configure your application _at runtime_, and one\nof the most effective places to do that is inside `config/runtime.exs` (available\nsince Elixir v1.11).\n\nThe `Dotenvy.source/2` function can accept a single file or a list of files.  When combined with `Config.config_env/0` it is easy to load up environment-specifc config, e.g.\n\n```elixir\nsource([\"#{config_env()}.env\", \"#{config_env()}.override.env\", System.get_env()])\n```\n\nBy default, the listed files do not _need_ to exist -- the function only needs to know where to look. This makes it easy to commit default values while still leaving the door open to developers to override values via their own configuration files.\n\nYou control if and how existing system env vars are handled: usually they should take precedence over values defined in `.env` files, so the `System.get_env()` should be included as the final input supplied to `source/2`.\n\nUnlike other packages, `Dotenvy` has no opinions about the names or locations of your dotenv config files, you just need to pass their paths to `Dotenvy.source/2` or `Dotenvy.source!/2`.\n\nFor a simple example, we can load a single file:\n\n```elixir\n# config/runtime.exs\nimport Config\nimport Dotenvy\n\nsource!([\".env\", System.get_env()])\n\nconfig :myapp, MyApp.Repo,\n    database: env!(\"DATABASE\", :string!),\n    username: env!(\"USERNAME\", :string),\n    password: env!(\"PASSWORD\", :string),\n    hostname: env!(\"HOSTNAME\", :string!),\n    pool_size: env!(\"POOL_SIZE\", :integer),\n    adapter: env!(\"ADAPTER\", :module, Ecto.Adapters.Postgres),\n    pool: env!(\"POOL\", :module?)\n```\n\nAnd then define your variables in the file(s) to be sourced. `Dotenvy` has no opinions about what you name your files; `.env` is merely a convention.\n\n```env\n# .env\nDATABASE=myapp_dev\nUSERNAME=myuser\nPASSWORD=mypassword\nHOSTNAME=localhost\nPOOL_SIZE=10\nPOOL=\n```\n\nWhen you set up your application configuration in this way, you are creating a contract with the environment: `Dotenvy.env!/2` will raise if the required variables have not been set or if the values cannot be properly transformed. This is an approach that works equally well for your day-to-day development and for mix releases.\n\nRead the [configuration strategies](docs/strategies.md) for more detailed examples of how to configure your app.\n\nRefer to the [\"dotenv\" (`.env`) file format](docs/dotenv-file-format.md) for more examples and features of the supported syntax.\n\nSee the `Dotenvy` module documentation on its functions.","ref":"readme.html#usage"},{"type":"extras","title":"Note for Mix Tasks - Dotenvy","doc":"If you have authored your own Mix tasks, you must ensure that they load the\napplication configuration in a way that is compatible with the runtime config.\nA good way to do this is to include `Mix.Task.run(\"app.config\")` inside the\n`run/1` implementation, e.g.\n\n```elixir\ndef run(_args) do\n  Mix.Task.run(\"app.config\")\n  # ...\nend\n```\n\nIf you are dealing with third-party mix tasks that fail to properly load configuration, you may need to manually call `mix app.config` before running them, e.g.\n\n```sh\nmix do app.config other.task\n```\n\nDefining a task `alias` in `mix.exs` is another way to accomplish this:\n\n```elixir\n# mix.exs\ndefp aliases do\n    [\n      \"other.task\": [\"app.config\", \"other.task\"]\n    ]\n```","ref":"readme.html#note-for-mix-tasks"},{"type":"extras","title":"Upgrading from v0.5.0 or before - Dotenvy","doc":"Starting with Dotenvy v0.6.0, the precedence of system env variables over parsed `.env` files is not defined; the `:overwrite?` and `:vars` options are no longer supported in `Dotenvy.source/2` and `Dotenvy.source!/2`. Instead, the `source` functions now accept file paths OR maps: this makes the question of variable precedence something that must be explicitly listed. The `source` functions act more like `Map.merge/2`, accumulating values, always giving precedence to the righthand source.\n\nMost users upgrading from v0.5.0 will wish to include `System.get_env()` as the final input to `source/2`.\n\n```elixir\n# in dotenvy 0.5.0 or before:\nsource([\"#{config_env()}.env\", \"#{config_env()}.override.env\"])\n\n# should be changed to the following in dotenvy 0.6.0:\nsource([\"#{config_env()}.env\", \"#{config_env()}.override.env\", System.get_env()])\n```\n\nIf you are relying on [variable interpolation](docs/dotenv-file-format.md) in your `.env` files, you may also need to include `System.get_env()` (or an equivalent subset) _before_ you list your `.env` files.  This is necessary to make values available to the file parser.\n\n```elixir\n# in dotenvy 0.5.0 or before:\nsource([\"#{config_env()}.env\", \"#{config_env()}.override.env\"])\n\n# should be changed to the following in dotenvy 0.6.0:\nsource([System.get_env(), \"#{config_env()}.env\", \"#{config_env()}.override.env\", System.get_env()])\n```\n\nThe change in syntax introduced in v0.6.0 favors a declarative list of sources over opaquely inferred inputs. This also opens the door for compatibility with other value sources, e.g. secure parameter stores.\n\n---------------------------------------------------\n\nImage Attribution: \"dot\" by Stepan Voevodin from the [Noun Project](https://thenounproject.com/)","ref":"readme.html#upgrading-from-v0-5-0-or-before"},{"type":"extras","title":"Cheatsheet","doc":"# Cheatsheet\n\nQuick reference for using Dotenvy.","ref":"cheatsheet.html"},{"type":"extras","title":"Setup - Cheatsheet","doc":"{: .col-3}","ref":"cheatsheet.html#setup"},{"type":"extras","title":"envs/ Directory - Cheatsheet","doc":"The recommended location for storing your `.env` files is inside a dedicated `envs/` directory.\nCode editors recognize the `.env` extension.\n\n#### Example envs/dev.env\n\n```env\nLOG_LEVEL=debug\nAWS_REGION=\"us-east-1\"\nHTTP_CLIENT=HTTPoison\n```","ref":"cheatsheet.html#envs-directory"},{"type":"extras","title":"Version Control - Cheatsheet","doc":"`envs/dev.env` is tracked by Git. Use it to store sensible non-sensitive defaults.\n\n`envs/dev.local.env` is ignored by Git. Use it to override any variables in the tracked version.\n\nThis must match the values used in the runtime configuration.\n\n#### .gitignore\n\n```\n*.local.env\n```","ref":"cheatsheet.html#version-control"},{"type":"extras","title":"Release Compatibility - Cheatsheet","doc":"For compatibility with [releases](https://hexdocs.pm/mix/Mix.Release.html), \nconfigure your builds to copy (i.e. \"overlay\") the contents of your `envs/` \ndirectory into the root of the release.\n\n#### mix.exs\n\n```elixir\ndefp releases do\n[\n    my_app: [\n    include_executables_for: [:unix],\n    steps: [:assemble, :tar],\n    overlays: [\"envs/\", \"priv/\", \"config/\"]\n    ]\n]\nend\n```","ref":"cheatsheet.html#release-compatibility"},{"type":"extras","title":"Runtime Config - Cheatsheet","doc":"Your `config/runtime.exs` is where you source your environment variables. \nThe last argument to `Dotenvy.source/2` or `Dotenvy.source!/2` takes precedence.\nIt's common to use `System.get_env()` as the final argument so any existing system\nenvironment variables will take precedence over anything parsed from the `.env` files.\n\n#### config/runtime.exs\n\n```elixir\nimport Config\nimport Dotenvy\n\ndir = System.get_env(\"RELEASE_ROOT\") || \"envs/\"\n# In an umbrella app:\n# dir = System.get_env(\"RELEASE_ROOT\") || Path.expand(\"./envs/\") <> \"/\"\n  \nsource!([\n  \"#{dir}#{config_env()}.env\",\n  \"#{dir}#{config_env()}.local.env\",\n  System.get_env()\n])\n```","ref":"cheatsheet.html#runtime-config"},{"type":"extras","title":"Example Database Configuration - Cheatsheet","doc":"{: .col-3}","ref":"cheatsheet.html#example-database-configuration"},{"type":"extras","title":"Dev ENV - Cheatsheet","doc":"#### envs/dev.env\n\n```\nPG_USERNAME=postgres\nPG_PASSWORD=postgres\nPG_HOSTNAME=localhost\nPG_PORT=5432\nPG_DATABASE=m_app_dev\nPG_POOL_SIZE=10\nPG_POOL=DBConnection.ConnectionPool\nPG_SSL=true\n```","ref":"cheatsheet.html#dev-env"},{"type":"extras","title":"Test ENV - Cheatsheet","doc":"#### envs/test.env\n\n```\nPG_USERNAME=postgres\nPG_PASSWORD=postgres\nPG_HOSTNAME=localhost\nPG_PORT=5432\nPG_DATABASE=my_app_test\nPG_POOL_SIZE=10\nPG_POOL=Ecto.Adapters.SQL.Sandbox\nPG_SSL=false\n```","ref":"cheatsheet.html#test-env"},{"type":"extras","title":"Runtime Configuration - Cheatsheet","doc":"#### config/runtime.exs\n\n```\nimport Config\nimport Dotenvy\n\ndir = System.get_env(\"RELEASE_ROOT\") || \"envs/\"\n  \nsource!([\n  \"#{dir}#{config_env()}.env\",\n  \"#{dir}#{config_env()}.local.env\",\n  System.get_env()\n])\n\nconfig :my_app, MyApp.PGRepo,\n  pool: env!(\"PG_POOL\", :module?),\n  pool_size: env!(\"PG_POOL_SIZE\", :integer),\n  database: env!(\"PG_DATABASE\", :string),\n  username: env!(\"PG_USERNAME\", :string),\n  password: env!(\"PG_PASSWORD\", :string),\n  port: env!(\"PG_PORT\", :integer),\n  hostname: env!(\"PG_HOSTNAME\", :string)\n```","ref":"cheatsheet.html#runtime-configuration"},{"type":"extras","title":"Transformations - Cheatsheet","doc":"System Environment variables are always stored as strings which may need to be \ntransformed into native Elixir data types.\n\n#### Used as the 2nd argument to `Dotenvy.env!/2` and `Dotenvy.env!/3`\n\n| Conversion Type   | Elixir Type   | On Empty String   |\n| --                | --            | --                |\n| `:atom`           | atom          | `:\"\"`             |\n| `:atom?`          | atom          | `nil`             |\n| `:atom!`          | atom          | raise ⚠           |\n| `:boolean`        | boolean       | `false`           |\n| `:boolean?`       | boolean       | `nil`             |\n| `:boolean!`       | boolean       | raise ⚠           |\n| `:charlist`       | charlist      | `''` i.e. `[]`    |\n| `:charlist?`      | charlist      | `nil`             |\n| `:charlist!`      | charlist      | raise ⚠           |\n| `:integer`        | integer       | `0`               |\n| `:integer?`       | integer       | `nil`             |\n| `:integer!`       | integer       | raise ⚠           |\n| `:float`          | float         | `0`               |\n| `:float?`         | float         | `nil`             |\n| `:float!`         | float         | raise ⚠           |\n| `:existing_atom`  | atom          | `:\"\"` or raise    |\n| `:existing_atom?` | atom          | `nil`             |\n| `:existing_atom!` | atom          | raise ⚠           |\n| `:module`         | atom          | `:\"Elixir.\"`      |\n| `:module?`        | atom          | `nil`             |\n| `:module!`        | atom          | raise ⚠           |\n| `:string`         | String        | `\"\"`              |\n| `:string?`        | String        | `nil`             |\n| `:string!`        | String        | raise ⚠           |\n\nCustom functions handle their own behavior.\n\n#### Custom Function Example\n\n```env\nPHX_IP=\"0, 0, 0, 0, 0, 0, 0, 0\"\n```\n\n```elixir\nconfig :feenix, FeenixWeb.Endpoint,\n    http: [\n      # Enable IPv6 and bind on all interfaces.\n      ip: env!(\"PHX_IP\", fn ip -> \n        ip \n        |> String.split(\",\") \n        |> Enum.map(&String.trim/1) \n        |> Enum.map(&String.to_integer/1) \n        |> List.to_tuple()\n      end)\n    ],\n```\n\nYour custom functions can raise a `Dotenvy.Error` to benefit from improved messages\nthat include helpful context about any problems, e.g.\n\n```elixir\nstrict_boolean! = fn\n  \"true\" -> true\n  \"false\" -> false\n  _ ->\n    raise Dotenvy.Error,\n      message: \"strict_boolean! values must be either true or false\"\nend\n\nconfig :myapp, :some_bool, env!(\"SOME_BOOL\", strict_boolean!)\n```\n\nThis will yield an error like the following:\n\n```\n** (RuntimeError) Error converting variable SOME_BOOL using custom function: strict_boolean! values must be either true or false\n```","ref":"cheatsheet.html#transformations"},{"type":"extras","title":"Strategies","doc":"# Strategies\n\nAlthough there are other places where `Dotenvy` may prove useful, it was designed with the `config/runtime.exs` in mind: most of the following use-cases will focus on that because it offers a clean and declarative way to load up the necessary variables.","ref":"strategies.html"},{"type":"extras","title":"A Note on Configuration Providers - Strategies","doc":"[Configuration providers](https://hexdocs.pm/elixir/Config.Provider.html) are most often invoked in the context of releases, and although they can solve certain problems that arise in production deployments, they tend to be an awkward fit for regular day-to-day development. `Dotenvy` seeks to normalize how configuration is loaded across environments, so having different methods depending on how you run your app is antithetical. We do not want some code that runs only in certain environments and not in others: it can make for untested or untestable code.\n\nSecondly, configuration providers sometimes shift the task of \"shaping\" the configuration out of Elixir and into some static representation (e.g. JSON or TOML). The allure of a straight-forward static file is deceiving because there is no easy way to delineate Elixir-specific subtleties such as distinguishing between keyword lists and maps. When configuration providers \"solve\" one problem, they often create another: it can require some busywork to convert values back into Elixir variable types that your application requires.\n\nFor these reasons, `Dotenvy` does not rely on [configuration providers](https://hexdocs.pm/elixir/Config.Provider.html); dotenv files are an easier \"lingua franca\".","ref":"strategies.html#a-note-on-configuration-providers"},{"type":"extras","title":"Dotenv for Dev and Prod - Strategies","doc":"The distinctions between \"dev\" and \"prod\" become less clear when we focus on configuration: ideally, the app is the same in all environments, it is only the configuration _values_ themselves that can be described as \"dev\" or \"prod\" -- in this example they will live inside a single `.env` file.\n\nLet's look at the three files that will make this work:\n\n### `config/config.exs`\n\n    # compile-time config\n    import Config\n\n    config :myapp,\n    ecto_repos: [MyApp.Repo]\n\n    config :myapp, MyApp.Repo,\n    migration_timestamps: [\n        type: :utc_datetime,\n        inserted_at: :created_at\n    ]\n\n### `config/runtime.exs`\n\n    import Config\n    import Dotenvy\n\n    source([\".env\", System.get_env()])\n\n    if config_env() == \"test\" do\n        config :myapp, MyApp.Repo,\n            database: \"myapp_test\",\n            username: \"test-user\",\n            password: \"test-password\",\n            hostname: \"localhost\",\n            pool_size: 10,\n            adapter: Ecto.Adapters.Postgres,\n            pool: Ecto.Adapters.SQL.Sandbox\n    else\n        config :myapp, MyApp.Repo,\n            database: env!(\"DATABASE\", :string!),\n            username: env!(\"USERNAME\", :string),\n            password: env!(\"PASSWORD\", :string),\n            hostname: env!(\"HOSTNAME\", :string!),\n            pool_size: env!(\"POOL_SIZE\", :integer),\n            adapter: env(\"ADAPTER\", :module, Ecto.Adapters.Postgres),\n            pool: env!(\"POOL\", :module?)    \n    end\n\n### `.env` (dev or prod)\n\n    DATABASE=myapp_dev\n    USERNAME=myuser\n    PASSWORD=mypassword\n    HOSTNAME=localhost\n    POOL_SIZE=10\n    POOL=\n\nThe `.env` shows some values suitable local development; if the app were deployed on a production box, it would be the same shape, but its values would point to a production database. For tests, values are hard-coded inside `runtime.exs`. This is one admittedly heavy-handed way to ensure that your test runs don't accidentally hit the wrong database, but it does mean that there is a small block of untestable code inside the if-statement.\n\nYou may notice that in this example we have done away with `config/dev.exs`, `config/test.exs`, and `config/prod.exs`. These should be used _only_ when your app has a legitimate compile-time need.  If you _can_ configure something at runtime, you _should_ configure it at runtime.  These extra config files are omitted to help demonstrate how the decisions about how the app should run can often be pushed into `runtime.exs`. This should help avoid confusion that often arises between compile-time and runtime configuration.","ref":"strategies.html#dotenv-for-dev-and-prod"},{"type":"extras","title":"Dotenvs for All Environments - Strategies","doc":"It is possible to use _only_ a `config.exs` and a `runtime.exs` file to configure\nmany Elixir applications: let the `.env` tell the app how to run!\nConsider the following setup:\n\n### `config/config.exs`\n\n    # compile-time config\n    import Config\n\n    config :myapp,\n    ecto_repos: [MyApp.Repo]\n\n    config :myapp, MyApp.Repo,\n    migration_timestamps: [\n        type: :utc_datetime,\n        inserted_at: :created_at\n    ]\n\n### `config/runtime.exs`\n\n    import Config\n    import Dotenvy\n\n    source([\".env\", \".env.\\#{config_env()}\", System.get_env()])\n\n    config :myapp, MyApp.Repo,\n        database: env!(\"DATABASE\", :string!),\n        username: env!(\"USERNAME\", :string),\n        password: env!(\"PASSWORD\", :string),\n        hostname: env!(\"HOSTNAME\", :string!),\n        pool_size: env!(\"POOL_SIZE\", :integer),\n        adapter: env(\"ADAPTER\", :module, Ecto.Adapters.Postgres),\n        pool: env!(\"POOL\", :module?)\n\n### `.env` (dev or prod)\n\n    DATABASE=myapp_dev\n    USERNAME=myuser\n    PASSWORD=mypassword\n    HOSTNAME=localhost\n    POOL_SIZE=10\n    POOL=\n\n### `.env.test`\n\n    DATABASE=myapp_test\n    USERNAME=myuser\n    PASSWORD=mypassword\n    HOSTNAME=localhost\n    POOL_SIZE=10\n    POOL=Ecto.Adapters.SQL.Sandbox\n\nThe above setup would likely commit the `.env.test` file so it was sure to override, and add `.env` to `.gitignore`, but other strategies are possible.  The above example demonstrates developer settings appropriate for local development in the sample `.env` file, but a production deployment would only differ in its _values_: the shape of the file would be the same.\n\nThe `.env.test` file is loaded when running tests, so its values override any of the\nvalues set in the `.env`.\n\nBy using `Dotenvy.env!/2`, a strong contract is created with the environment: the\nsystem running this app _must_ have the designated environment variables set somehow,\notherwise this app will not start (and a specific error will be raised).\n\nUsing the nil-able variants of the type-casting (those ending with `?`) is an easy\nway to fall back to `nil` when the variable contains an empty string: `env!(\"POOL\", :module?)` requires that the `POOL` variable is set, but it will return a `nil` if the value is an empty string.\n\nSee `Dotenvy.Transformer` for more details.","ref":"strategies.html#dotenvs-for-all-environments"},{"type":"extras","title":"Releases - Strategies","doc":"One of the hurdles when dealing with Elixir releases is that only certain files are packaged into them.  One solution to this is to specify additional directories to include in the release via the `overlays` option in your `mix.exs`, e.g. an `envs/` directory which contains your dotenv files:\n\n    ```elixir\n    # mix.exs\n    defp releases do\n        [\n        myapp: [\n            overlays: [\"envs/\"]\n        ]\n        ]\n    end\n    ```\n\nSince these files are copied to the root of your release, the relative paths used in your `runtime.exs` will not be able to find them when your app is running in the context of a release. One solution to this is to rely on the `RELEASE_ROOT` system environment variable which is set when a release is run. If this value exists, it will represent the fully qualified path to your release; this variable will not be set when running your app locally (e.g. during development).\n\nWe can use the presence of the `RELEASE_ROOT` to determine a directory prefix for where to look for our dotenv files, e.g.:\n\n    ```elixir\n    import Config\n    import Dotenvy\n\n    # For local development, read dotenv files inside the envs/ dir;\n    # for releases, read them at the RELEASE_ROOT\n    config_dir_prefix =\n    System.fetch_env(\"RELEASE_ROOT\")\n    |> case do\n        :error ->\n        \"envs/\"\n\n        {:ok, value} ->\n        IO.puts(\"Loading dotenv files from #{value}\")\n        \"#{value}/\"\n    end\n\n    source!([\n    \"#{config_dir_prefix}.env\",\n    \"#{config_dir_prefix}.#{config_env()}.env\",\n    \"#{config_dir_prefix}.#{config_env()}.local.env\"\n    System.get_env()\n    ])\n    ```\n\nOr more succinctly:\n\n    ```elixir\n    config_dir_prefix = System.get_env(\"RELEASE_ROOT\") || \"envs/\"\n    ```\n\nIt can be safer to reference an absolute path, e.g.\n\n    ```elixir\n    config_dir_prefix = System.get_env(\"RELEASE_ROOT\") || Path.expand(\"./envs/\") <> \"/\"\n    ```\n\nThis is especially important when working with umbrella apps (see below).","ref":"strategies.html#releases"},{"type":"extras","title":"Umbrella Apps - Strategies","doc":"Elixir [Umbrella Projects](https://elixir-lang.org/getting-started/mix-otp/dependencies-and-umbrella-projects.html) consume configuration slightly differently due to how they are organized.\n\nIn particular, you have to be very careful about relative paths when working in an umbrella project.  Depending on what you're doing, the path may be _relative to a single application_ instead of relative to the root of the repository. Using `Path.expand/1` is a good way to anchor your `config/runtime.exs` to point to the root of the repository instead of it resolving to the root of a specific application within the umbrella. E.g.\n\n    ```elixir\n    env_dir_prefix = System.get_env(\"RELEASE_ROOT\") || Path.expand(\"./envs/\") <> \"/\"\n\n    source!([\n        \"#{env_dir_prefix}#{config_env()}.env\",\n        \"#{env_dir_prefix}#{config_env()}.local.env\",\n        System.get_env()\n    ])\n    ```","ref":"strategies.html#umbrella-apps"},{"type":"extras","title":"Dotenv File Format","doc":"# Dotenv File Format\n\n`.env` files (a.k.a. \"dotenv\") store key-value pairs in a format descended from\nsimple bash files that exported environment variables.\n\nThis implementation cleaves closely to the format described by the original [dotenv](https://github.com/bkeepers/dotenv) package, but it is not a direct match (by design).\n\nTypically, a dotenv (`.env`) file is formatted into simple key-value pairs:\n\n    S3_BUCKET=YOURS3BUCKET\n    SECRET_KEY=YOURSECRETKEYGOESHERE\n\nOptionally, you may add `export` in front of each line so you can `source` the file in Bash:\n\n    export S3_BUCKET=YOURS3BUCKET\n    export SECRET_KEY=YOURSECRETKEYGOESHERE","ref":"dotenv-file-format.html"},{"type":"extras","title":"Variable Names - Dotenv File Format","doc":"For the sake of portability (and sanity), environment variable names must consist solely of letters, digits, and the underscore ( `_` ) and must not begin with a digit. In regex-speak, the names must match the following pattern:\n\n    [a-zA-Z_]+[a-zA-Z0-9_]*","ref":"dotenv-file-format.html#variable-names"},{"type":"extras","title":"Example variable names - Dotenv File Format","doc":"DATABASE_URL  # ok  \n    foobar        # ok  \n    NO-WORK       # <-- invalid !!!\n    ÜBER          # <-- invalid !!!\n    2MUCH         # <-- invalid !!!","ref":"dotenv-file-format.html#example-variable-names"},{"type":"extras","title":"Values - Dotenv File Format","doc":"Values are to the right of the equals sign. They may be quoted.\nUsing single quotes will prevent variables from being interpolated.\n\n    SIMPLE=xyz123\n    INTERPOLATED=\"Multiple\\nLines and variable substitution: ${SIMPLE}\"\n    NON_INTERPOLATED='raw text without variable interpolation'\n    MULTILINE = \"\"\"\n    long text here,\n    e.g. a private SSH key\n    \"\"\"","ref":"dotenv-file-format.html#values"},{"type":"extras","title":"Escape Sequences - Dotenv File Format","doc":"The following character strings will be interpreted (i.e. escaped) as specific codepoints in the same way you would expect if the values were assigned inside a script. Remember: when a text file is read, it is read as a series of utf8 encoded code points. Character sequences like `\\n` have no special meaning until they are \"escaped\" and a combination of codepoints is replaced by a single codepoint.\n\n- `\\n` Linefeed (aka newline); `<<92, 110>>` -> `<<10>>`\n- `\\r` Carriage return; `<<92, 114>>` -> `<<13>>`\n- `\\t` Tab; -> `<<92, 116>>` -> `<<9>>`\n- `\\f` Form feed; -> `<<92, 102>>` -> `<<12>>`\n- `\\b` Backspace; -> `<<92, 98>>` -> `<<8>>`\n- `\\\"` Double-quote; ->  `<<92, 34>>` -> `<<34>>`\n- `\\'` Single-quote; -> `<<92, 39>>` -> `<<39>>`\n- `\\\\` Backslash; -> `<<92, 92>>` -> `<<92>>`\n- `\\uFFFF` Unicode escape (4 hex characters to denote the codepoint)\n\nIf a backslash precedes any other character, that character will be interpreted literally: i.e. the backslash will be ignored and removed from output.","ref":"dotenv-file-format.html#escape-sequences"},{"type":"extras","title":"Interpolation (a.k.a. Variable Substitution) - Dotenv File Format","doc":"Values left unquoted or wrapped in double-quotes will interpolate variables in the `${VAR}` syntax. This can be useful for referencing existing system environment variables or to reference variables previously parsed.\n\nFor example:\n\n    USER=admin\n    EMAIL=${USER}@example.org\n    DATABASE_URL=\"postgres://${USER}@localhost/my_database\"\n    CACHE_DIR=${PWD}/cache\n\nMulti-line values (e.g. private keys) can use the triple-quoted heredoc syntax:\n\n    PRIVATE_KEY=\"\"\"\n    -----BEGIN RSA PRIVATE KEY-----\n    ...\n    HkVN9...\n    ...\n    -----END DSA PRIVATE KEY-----\n    \"\"\"","ref":"dotenv-file-format.html#interpolation-a-k-a-variable-substitution"},{"type":"extras","title":"Non-Interpolated - Dotenv File Format","doc":"If your values must retain `${}` in their output, wrap the value in single quotes, e.g.:\n\n    PASSWORD='!@G0${k}k'\n    MESSAGE_TEMPLATE='''\n        Hello ${PERSON},\n\n        Nice to meet you!\n    '''","ref":"dotenv-file-format.html#non-interpolated"},{"type":"extras","title":"Comments - Dotenv File Format","doc":"The hash-tag `#` symbol denotes a comment when on its own line or when it follows a quoted value.  It is not treated as a comment when it appears within quotes.\n\n    # This is a comment\n    SECRET_KEY=YOURSECRETKEYGOESHERE # also a comment\n    SECRET_HASH=\"something-with-a-hash-#-this-is-not-a-comment\"","ref":"dotenv-file-format.html#comments"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nThe format is based on [Keep a Changelog](https://keepachangelog.com/en/1.0.0/),\nand this project adheres to [Semantic Versioning](https://semver.org/spec/v2.0.0.html).","ref":"changelog.html"},{"type":"extras","title":"v0.9.0 - Changelog","doc":"- Removes fallback to `System.fetch_env/1` and instead requires explicit sourcing of system envs. See [Issue 21](https://github.com/fireproofsocks/dotenvy/issues/21)\n- Bumps Elixir version specified for local development in `.tool-versions`\n- Updates dependencies to latest","ref":"changelog.html#v0-9-0"},{"type":"extras","title":"v0.8.0 - Changelog","doc":"- Enables exception rescuing to report on problems with custom callback functions\n- Moves `Dotenvy.Transformer.Error` to `Dotenvy.Error` to offer a simpler interface\n  for devs who want to raise errors from custom transformer functions\n- Improved documentation and examples for usage in umbrella apps\n- Updates all internal options in the parser to use `%Opts{}` struct\n- Improves test coverage\n- Bumps Elixir version specified for local development in `.tool-versions`\n- Updates dependencies to latest","ref":"changelog.html#v0-8-0"},{"type":"extras","title":"v0.7.0 - Changelog","doc":"- Formally defines a type for all supported conversions to improve documentation and specs\n- Updates dependencies to latest\n- Specifies Elixir 1.13 as required (simply because I can't get anything older to compile)","ref":"changelog.html#v0-7-0"},{"type":"extras","title":"v0.6.0 - Changelog","doc":"- Does away with the confusing `:overwrite?` and `vars` options in favor of a simple declarative/explicit inputs. `source/2` now accepts ad-hoc maps as inputs.\n- Updates dependencies including `:ex_doc` to take advantage of admonishment blocks.\n- Various documentation cleanups/clarifications.","ref":"changelog.html#v0-6-0"},{"type":"extras","title":"v0.5.0 - Changelog","doc":"- Shifts storage of system environment variables to the application process dictionary and alters the reading of this data to help improve the security posture and avoid leaking env values. `:side_effect` option for `source/2` and `source!/2` function changed.","ref":"changelog.html#v0-5-0"},{"type":"extras","title":"v0.4.1 - Changelog","doc":"- Makes error messages more informative when unable to convert strings to integers or floats","ref":"changelog.html#v0-4-1"},{"type":"extras","title":"v0.4.0 - Changelog","doc":"- Adds support for custom transformer types by allowing an arity 1 function as the second argument to Dotenvy.Transformer.to/2. See [Issue 2](https://github.com/fireproofsocks/dotenvy/issues/2)","ref":"changelog.html#v0-4-0"},{"type":"extras","title":"v0.3.0 - Changelog","doc":"- Renames Dotenvy.Transformer.to/2 to `Dotenvy.Transformer.to!/2` to better communicate that it may raise an error.\n- Returns key name in errors for easier troubleshooting.\n- Tracks an error if the `:require_files` option lists a file not included in the `files` input (for sanity).\n- Introduces `Dotenvy.env!/3` (which is the same as `Dotenvy.env/3` but with no defaults provided). This better communicates that it may raise an error (because internally it relies on `Dotenvy.Transformer.to!/2`)\n- Deprecates `Dotenvy.env/3` in favor of `Dotenvy.env!/3`","ref":"changelog.html#v0-3-0"},{"type":"extras","title":"v0.2.0 - Changelog","doc":"Adds support for default `type` of `:string` to the `Dotenvy.env!/2` and `Dotenvy.env/3` functions.","ref":"changelog.html#v0-2-0"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"Initial release.","ref":"changelog.html#v0-1-0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}